#!/bin/bash

CONTAINER=$1
IP=$2
GATEWAY=$3

echo "
======= Stage 1, making container ======="
NETPLAN_FILE="
network:
  version: 2
  ethernets:
    eth0:
      addresses: [ $IP ]
      gateway4: $GATEWAY
      nameservers:
        addresses: [ 8.8.8.8, 4.2.2.2 ]
"
lxc launch ubuntu:18.04 $CONTAINER
sleep 3
lxc exec $CONTAINER -- systemctl disable sshd
# lxc exec $CONTAINER -- bash -c "echo \"$NETPLAN_FILE\" > /etc/netplan/50-cloud-init.yaml"
lxc restart $CONTAINER
sleep 3

echo "
======= Stage 2, prepare container ======="
lxc exec $CONTAINER -- deluser ubuntu
lxc exec $CONTAINER -- apt-get update
lxc exec $CONTAINER -- apt-get -y upgrade

echo "
======= Stage 3, install HAProxy, Certbot and nginx for it ======="
lxc exec $CONTAINER -- apt-get -y install haproxy certbot nginx

echo "
======= Stage 4, prepare nginx for certbot ======="
ACME_WEBROOT="/var/www/acme";
NGINX_FILE="
user www-data;
worker_processes auto;
pid /run/nginx.pid;

events {
    worker_connections 768;
}

http {
    include /etc/nginx/mime.types;
    default_type application/octet-stream;

	server {
		listen	 127.0.0.1:8081  default_server;
		server_name  _;
		location / {
			root $ACME_WEBROOT;
		}
	}
}
"
lxc exec $CONTAINER -- bash -c "echo \"$NGINX_FILE\" > /etc/nginx/nginx.conf"
lxc exec $CONTAINER -- mkdir $ACME_WEBROOT

echo "
======= Stage 5, setup certbot hooks ======="
PRE_VALIDATION_SCRIPT='#!/bin/sh
systemctl start nginx
'
POST_VALIDATION_SCRIPT='#!/bin/sh
systemctl stop nginx
'
DEPLOY_SCRIPT='#!/bin/sh
SSL_DIR=/etc/ssl/private
pem_file="$SSL_DIR/${RENEWED_DOMAINS}.pem"
echo "" > $pem_file
chmod g=,o= $pem_file
cat $RENEWED_LINEAGE/privkey.pem >> $pem_file
cat $RENEWED_LINEAGE/fullchain.pem >> $pem_file
systemctl reload haproxy
'
HOOKS_DIR="/etc/letsencrypt/renewal-hooks"

for dir in $HOOKS_DIR $HOOKS_DIR/pre $HOOKS_DIR/post $HOOKS_DIR/deploy; do
  lxc exec $CONTAINER -- mkdir $dir
done

lxc exec $CONTAINER -- bash -c "echo \"$PRE_VALIDATION_SCRIPT\" > $HOOKS_DIR/pre/1.sh"
lxc exec $CONTAINER -- chmod u+x $HOOKS_DIR/pre/1.sh
lxc exec $CONTAINER -- bash -c "echo \"$POST_VALIDATION_SCRIPT\" > $HOOKS_DIR/post/1.sh"
lxc exec $CONTAINER -- chmod u+x $HOOKS_DIR/post/1.sh
lxc exec $CONTAINER -- bash -c "echo \"$DEPLOY_SCRIPT\" > $HOOKS_DIR/deploy/1.sh"
lxc exec $CONTAINER -- chmod u+x $HOOKS_DIR/deploy/1.sh

echo "
======= After this installation:

1) Check for absence of obvious errors in above outputs.

2) Add to /etc/haproxy/haproxy.cfg following rules to direct acme challenges to nginx backend:
'''
frontend http-in
        bind *:80
        acl is_acme path_beg /.well-known/acme-challenge/
        use_backend acme if is_acme
backend acme
        server acme1 127.0.0.1:8081
'''

3) Use certbot with webroot plugin for authentication, with path arguments -w $ACME_WEBROOT

4) And run two commands (this isn't automated yet into cron):
'''
lxc exec $CONTAINER -- systemctl stop ngnix
lxc exec $CONTAINER -- systemctl disable ngnix
'''
"

